import sys
import ruamel.yaml

from ruamel.yaml.comments import CommentedMap as cmap

from pathlib import Path

class MyComposer(ruamel.yaml.composer.Composer):
    def compose_scalar_node(self, anchor):
        event = self.parser.get_event()
        tag = self.resolver.resolve(ruamel.yaml.nodes.ScalarNode, event.value, event.implicit)
        #print(f'{event} == {tag}')
        node = ruamel.yaml.nodes.ScalarNode(
            tag,
            event.value,
            event.start_mark,
            event.end_mark,
            style=event.style,
            comment=event.comment,
            anchor=anchor,
        )
        if anchor is not None:
            self.anchors[anchor] = node
        return node

loader = ruamel.yaml.YAML(typ='safe', pure=True)
# should ignore !add and !remove, but does not
loader.Composer = MyComposer
# don't sort keys (preserve dict order)
loader.Representer = ruamel.yaml.RoundTripRepresenter 
# don't crash on duplicate anchors
loader.allow_duplicate_keys = True

# 'True' to have everything flow style (single line, with brackets for lists and maps)
# 'False' to have everything block style (everything on its own line with '-' for new list element and indents)
# 'Not set' to have leaf collections in flow, and parent collections in blocks
#   (ternary logic boolean ffs)
loader.default_flow_style = False


#loader.preserve_quotes = True

# output '~' for None
loader.emitter.alt_null = '~'


loader.indent(mapping=2, sequence=4, offset=2)

# ignore !remove 
@loader.register_class
class Remove:
    yaml_tag = '!remove'

    def __init__(self, s):
        self._s = s

    @classmethod
    def from_yaml(cls, constructor, node):
        return cls(node.value)

    def __repr__(self):
        return repr(self._s)
        
# ignore !add
@loader.register_class
class Add:
    yaml_tag = '!add'

    def __init__(self, s):
        self._s = s

    @classmethod
    def from_yaml(cls, constructor, node):
        return cls(node.value)

    def __repr__(self):
        return repr(self._s)


# <user>/<mods>/<thismodname>/gen
mods_dir = Path(r'..\..')
ruleset_dir = mods_dir / r'XComFiles\Ruleset'

track_to_musics = {}
with open( ruleset_dir / 'music_XCOMFILES.rul', 'r', encoding='utf-8') as music_file:
    music_file_string = music_file.read().replace('\t', ' ')
    content = loader.load(music_file_string)
    
    track_to_musics = { track_definition['type']: track_definition for track_definition in content.get('musics', []) if 'type' in track_definition }
    
tracks_used_in_deployments = set()
tracks_used_in_briefings = set()
with open( ruleset_dir / 'alienDeployments_XCOMFILES.rul', 'r', encoding='utf-8') as deployments_file:
    deployments_file_string = deployments_file.read().replace('\t', ' ')
    content = loader.load(deployments_file_string)

    for deployment in content.get('alienDeployments', []):
        if 'delete' in deployment:
            continue

        tracks_used_in_deployments.update(deployment.get('music', []))
        
        if 'briefing' in deployment and 'music' in deployment['briefing']:
            tracks_used_in_briefings.add(deployment['briefing']['music'])

print(f'tracks used in both deployments and briefings:')
print(tracks_used_in_briefings & tracks_used_in_deployments)
print(f'---')

tracks_that_need_updating = set((track for track in tracks_used_in_deployments if 'GMTAC' not in track))

for track in list(tracks_that_need_updating):
    if track not in track_to_musics:
        print(f'!!! track {track} is used in some deployment, but not present in musics')
        tracks_that_need_updating.discard(track)

def make_battlescaped_track_definition(definition):
    new_definition = {
        'type': definition['type'] + "_GMTAC",
        'name': definition['name'] if 'name' in definition else definition['type'],
    }
    
    for key in definition.keys():
        if key not in {'type', 'name'}:
            new_definition[key] = definition[key]
    
    return new_definition
    
    
updated_track_definitions = []
for type, definition in ((key, value) for key, value in track_to_musics.items() if key in tracks_that_need_updating): # use track_to_musics to keep entries order
    delete_definition = cmap({
        'delete': type
    })
    if len(updated_track_definitions) > 0:
        delete_definition.yaml_set_start_comment('\n')
        
    updated_track_definitions.append(delete_definition)

    new_definition = make_battlescaped_track_definition(definition)
    updated_track_definitions.append(new_definition)
    
result = {
    'musics': updated_track_definitions
}

with open( 'modified_musics.rul', 'w', encoding='utf-8') as out:
    loader.dump(result, out)
    
### make full music_XCOMFILES.rul file just in case

all_tracks = []
for type, definition in track_to_musics.items():
    if type not in tracks_that_need_updating:
        all_tracks.append(definition)
    else:
        all_tracks.append(make_battlescaped_track_definition(definition))
        
        
result = {
    'musics': all_tracks
}

with open( 'music_XCOMFILES.rul-example', 'w', encoding='utf-8') as out:
    loader.dump(result, out)